using DutchTreat.Data.Entities;
using System.Collections.Generic;

namespace DutchTreat.Data
{
    public interface IDutchRepository
    {
        IEnumerable<Product> GetAllProducts();
        IEnumerable<Product> GetProductsByCategory(string category);
        IEnumerable<Order> GetAllOrders(bool includedItems);
        IEnumerable<Order> GetAllOrdersByUser(string username, bool includedItems);
        Order GetOrderById(string username, int id);
        bool SaveAll();
        void AddEntity(Order model);
    }
}